using Microsoft.CodeAnalysis;
using Seenginx.Models;
using Seenginx.Services.Models;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.AccessControl;
using System.Text;
using System.Threading.Tasks;

namespace Seenginx.Services
{
	public class NginxService : INginxService
	{
		private readonly ConfigPaths ConfigPaths;

		public NginxService(ConfigPaths configPaths)
		{
			ConfigPaths = configPaths;
		}

		public async Task<Result<bool>> ValidateNewConfigurationAsync(NewFileForm newFileForm)
		{
			var validationResult = new Result<bool>();
			try
			{
				var filePath = Path.Combine(ConfigPaths.NginxPath, "conf.d", $"{newFileForm.Name}.conf");

				if (File.Exists(filePath))
					return validationResult.Invalidate($"There's already a file with the '{newFileForm.Name}.conf' name.");

				return validationResult;
			}
			catch (Exception ex)
			{
				return validationResult.Invalidate($"Exception at {nameof(ValidateNewConfigurationAsync)}()", ex);
			}
		} 

		public async Task<Result<ConfigFile>> AddFileAsync(NewFileForm newFileForm)
		{
			var addResult = new Result<ConfigFile>();
			try
			{

				var newFile = new ConfigFile();
				newFile.Name = $"{newFileForm.Name}.conf";
				newFile.Folder = "/conf.d";
				newFile.FullPath = Path.Combine(ConfigPaths.NginxPath, "conf.d", newFile.Name);
				newFile.Body = newFileForm.SelectedTemplate == 0.ToString() ? string.Empty : (await GetTemplates()).SingleOrDefault(t => t.Name == newFileForm.SelectedTemplate)?.Code;
				newFile.LastUpdated = DateTime.UtcNow;

				await File.WriteAllTextAsync(newFile.FullPath, newFile.Body, Encoding.UTF8);

				addResult.SetData(newFile);

				return addResult;
			}
			catch (Exception ex)
			{
				return addResult.Invalidate(ex.Message, ex);
			}
		}

		public async Task<IEnumerable<ConfigFile>> GetFilesAsync()
		{
			await Task.Run(() => { });
			var rootConfigs = Directory.GetFiles(ConfigPaths.NginxPath, "*.conf");
			var confdConfigs = Directory.GetFiles(Path.Combine(ConfigPaths.NginxPath, "conf.d"), "*.conf");
			var sitesAvailableConfigs = Directory.GetFiles(Path.Combine(ConfigPaths.NginxPath, "sites-available"), "*.conf");
			var sitesEnabledConfigs = Directory.GetFiles(Path.Combine(ConfigPaths.NginxPath, "sites-enabled"), "*.conf");

			var rootConfigFiles = rootConfigs.Select(fp =>
			{
				var fileName = Path.GetFileName(fp);
				var configFile = new ConfigFile();
				configFile.CanBeDeleted = fileName != "nginx.conf";
				configFile.Folder = "/";
				configFile.LastUpdated = File.GetLastWriteTime(fp);
				configFile.Name = fileName;
				configFile.FullPath = Path.Combine(ConfigPaths.NginxPath, configFile.Name);
				configFile.Body = File.ReadAllText(fp);
				return configFile;
			});
			var confdConfigFiles = confdConfigs.Select(fp =>
			{
				var fileName = Path.GetFileName(fp);
				var configFile = new ConfigFile();
				configFile.CanBeDeleted = true;
				configFile.Folder = "/conf.d";
				configFile.LastUpdated = File.GetLastWriteTime(fp);
				configFile.Name = fileName;
				configFile.FullPath = Path.Combine(ConfigPaths.NginxPath, configFile.Folder.Replace("/", string.Empty), configFile.Name);
				configFile.Body = File.ReadAllText(fp);
				return configFile;
			});
			var sitesAvailableConfigFiles = sitesAvailableConfigs.Select(fp =>
			{
				var fileName = Path.GetFileName(fp);
				var configFile = new ConfigFile();
				configFile.CanBeDeleted = true;
				configFile.Folder = "/sites-available";
				configFile.LastUpdated = File.GetLastWriteTime(fp);
				configFile.Name = fileName;
				configFile.FullPath = Path.Combine(ConfigPaths.NginxPath, configFile.Folder.Replace("/", string.Empty), configFile.Name);
				configFile.Body = File.ReadAllText(fp);
				return configFile;
			});
			var sitesEnabledConfigFiles = sitesEnabledConfigs.Select(fp =>
			{
				var fileName = Path.GetFileName(fp);
				var configFile = new ConfigFile();
				configFile.CanBeDeleted = true;
				configFile.Folder = "/sites-enabled";
				configFile.LastUpdated = File.GetLastWriteTime(fp);
				configFile.Name = fileName;
				configFile.FullPath = Path.Combine(ConfigPaths.NginxPath, configFile.Folder.Replace("/", string.Empty), configFile.Name);
				configFile.Body = File.ReadAllText(fp);
				return configFile;
			});
			var finalList = new List<ConfigFile>();
			finalList.AddRange(rootConfigFiles);
			finalList.AddRange(confdConfigFiles);
			finalList.AddRange(sitesAvailableConfigFiles);
			finalList.AddRange(sitesEnabledConfigFiles);

			return finalList;
		}

		public async Task<IEnumerable<Template>> GetTemplates()
		{
			var templates = new List<Template>();
			try
			{
				var nginxTemplateDirectory = Path.Combine(Directory.GetCurrentDirectory(), "wwwroot", "templates", "nginx");
				var nginxTemplateFiles = Directory.GetFiles(nginxTemplateDirectory, "*.template");

				foreach (var templateFilePath in nginxTemplateFiles)
				{
					var template = new Template();
					var templateFileLines = await File.ReadAllLinesAsync(templateFilePath);
					template.Name = templateFileLines.FirstOrDefault();
					template.Code = string.Join(Environment.NewLine, templateFileLines.Skip(2));
					templates.Add(template);
				}

				return templates;
			}
			catch (Exception ex)
			{
				throw ex;
			}
		}

		public async Task<Result<string>> TestNginxConfigurations(ConfigFile configFile)
		{
			await Task.Run(() => { });
			var result = new Result<string>();
			result.SetData("Uhu");
			return result;
		}

		public async Task<Result<bool>> DeleteConfigurationFileAsync(ConfigFile configFile)
		{
			var result = new Result<bool>(true);
			try
			{
				if (!File.Exists(configFile.FullPath))
					return result.Invalidate($"File '{configFile.FullPath}' not found.");

				File.Delete(configFile.FullPath);

				return result;
			}
			catch (Exception ex)
			{
				return result.Invalidate(ex.Message, ex);
			}
		}



		//public async Task<Result<bool>> DeleteFileAsync(ConfigFile configFile)
		//{
		//	var result = new Result<bool>(true);
		//	try
		//	{
		//		return result;
		//	}
		//	catch (Exception ex)
		//	{
		//		return result.Invalidate(ex.Message, ex);
		//	}
		//}
	}
}
